/*
 *
 * Edscott Wilson Garcia 2001-2003 for xfce project.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/stat.h>

#include <errno.h>
#include <limits.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <glib.h>

#include <libxml/parser.h>
#include <libxml/tree.h>

#define XML_NS XML_XML_NAMESPACE
#define XFCE_NS "http://www.xfce.org"
#define XFCE_NS_PREFIX "xfce"

static GHashTable *translation_hash = NULL;
static GHashTable *application_hash = NULL;
static GHashTable *glob_hash = NULL;
static GHashTable *types_hash = NULL;
static GHashTable *repeat_app_hash = NULL;

typedef struct magic_t {
	char *type;
	char *offset;
	char *value;
	char *mask;
}magic_t;

typedef struct mime_t {
	char *mimetype;
	char **apps;
	char *magic_priority;
	magic_t **magic;
	char *icon;
}mime_t;


xmlNodePtr root;
xmlNsPtr    xfceNS;


static void destroy_application_hash(GHashTable *hash){
    if (!hash) return;
   // g_hash_table_foreach(hash, clear_application, NULL);
    g_hash_table_destroy(hash);
}


void add2types(char *key)
{
	  mime_t *mt;
	  //printf("glob pattern=%s ",value);
	  //if (*value != '*') printf("glob pattern=%s ",value); 
	  mt=(mime_t *)g_hash_table_lookup(types_hash,(char *)key);
	  if (mt){
             //g_warning("Value already in types hash %s",(char *)key);
          }
  	  else {
	     mt =g_new0(mime_t,1);
	     mt->mimetype=g_strdup(key);
	     mt->apps= (char **)malloc(1*sizeof(char *));
	     *(mt->apps)=NULL;
	     mt->icon=NULL;
	     mt->magic_priority=0;
	     mt->magic= (magic_t **)malloc(1*sizeof(magic_t *));
	     *(mt->magic)=NULL;
	     
             g_hash_table_insert(types_hash, 
			     (gpointer) g_strdup(key),(gpointer) mt);

	  }
}

void output(gpointer key, gpointer value, gpointer user_data){
    int i;
    xmlNodePtr node,subnode;
    mime_t *mt=(mime_t *)value;
    node = xmlNewTextChild(root, NULL, "mime-type", NULL);
    xmlSetProp(node,"type", (char *)key);
    if (!mt || !mt->icon) {
	    //printf("%s.... no icon!\n",(char *)key);
    } else  xmlSetProp(node,"icon",mt->icon);
    g_free(mt->icon); mt->icon=NULL;
}


void reg_build_list(void){
	mime_t *mt;
 xmlChar *mimetype;
 xmlChar *key;
 xmlChar *value;
 xmlChar *type;
 xmlChar *offset;
 xmlChar *mask;
 xmlNodePtr node;
 xmlNodePtr ssnode;
 xmlNodePtr subnode;
 xmlDocPtr doc;
 gchar *mimefile=NULL;
 char *files[]={
	"../freedesktop.org.xml",
	"../xfce.org.xml",
	NULL
 };
 char *themes[]={
	 "Crystal",
	 "FreeIcons",
	 "gnome",
	 "industrial",
	 "noia",
	 "plain",
	 "xfce",
	 "no_icons",
	 NULL
 };
	 
 int i,jj;
    
#ifdef DEBUG
            printf("DBG: reg_build_list\n");
#endif
 if (application_hash){
    destroy_application_hash(application_hash);
    application_hash=NULL;
 }
 if (translation_hash){
    destroy_application_hash(translation_hash);
    translation_hash=NULL;
 }
 if (glob_hash){
    destroy_application_hash(glob_hash);
    glob_hash=NULL;
 }
 if (types_hash){
    destroy_application_hash(types_hash);
    types_hash=NULL;
 }

 translation_hash = g_hash_table_new(g_str_hash, g_str_equal);
 glob_hash = g_hash_table_new(g_str_hash, g_str_equal);
 types_hash = g_hash_table_new(g_str_hash, g_str_equal);




 for (i=0;files[i];i++){
    mimefile=g_strconcat(files[i],NULL);
    if (access(mimefile,R_OK)!=0){
            printf("DBG1: access(%s,R_OK)!=0\n",mimefile);
	g_free(mimefile); mimefile=NULL;
	exit(1);
    }
    xmlKeepBlanksDefault(0);

    if((doc = xmlParseFile(mimefile)) == NULL)
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
	exit(1);
    }
	
    node = xmlDocGetRootElement(doc);
    //xfceNS = xmlNewNs (node,"http://www.xfce.org/","xfce");

    if(!xmlStrEqual(node->name, (const xmlChar *)"mime-info"))
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
        xmlFreeDoc(doc);
	exit(1);
    }
   /* Now parse the xml tree */
            printf("DBG: parsing %s\n",mimefile);
    for(node = node->children; node; node = node->next)
    {
	gboolean glob_present=FALSE; 	    
	    //printf("name=%s\n",node->name);
	if(xmlStrEqual(node->name, (const xmlChar *)"mime-type") )
	{
	    mimetype = xmlGetProp(node, (const xmlChar *)"type");
	    //if (!mimetype) mimetype = xmlGetNsProp (node, (const xmlChar *)"type","xfce");
	    if (!mimetype) {
		    continue;
	    }
	    add2types(g_strdup(mimetype));
	    //printf("%s->",mimetype);
    	    
	    for(subnode = node->children; subnode; subnode = subnode->next){
	        char *lang2;
		if(xmlStrEqual(subnode->name, (const xmlChar *)"glob")){
		   glob_present=TRUE;	
	           key = xmlGetProp(subnode, (const xmlChar *)"pattern");
		   if (key){
			  mime_t *p;
			  //printf("glob pattern=%s ",value);
			  //if (*value != '*') printf("glob pattern=%s ",value);
			  p=(mime_t *)g_hash_table_lookup(glob_hash,(char *)key);
			  if (p){
                             g_warning("Overwriting value already in glob hash for %s %s->%s\n",
					     (char *)key,p->mimetype,mimetype);
			     g_free(p->mimetype);
			     p->mimetype=g_strdup(mimetype);
			     // allow key to leak from memory...
			     g_hash_table_replace(glob_hash,(gpointer) g_strdup(key),(gpointer) p);
		          }
		  	  else {
			     mt =g_new0(mime_t,1);
			     mt->mimetype=g_strdup(mimetype);
	     		     mt->apps= (char **)malloc(1*sizeof(char *));
	     		     *(mt->apps)=NULL;
			     mt->icon=NULL;
	                     g_hash_table_insert(glob_hash, 
					     (gpointer) g_strdup(key),(gpointer) mt);
			  }
		    	  xmlFree(key);
		   }
		}
		if(xmlStrEqual(subnode->name, (const xmlChar *)"magic")){
		   mime_t *p=(mime_t *)g_hash_table_lookup(types_hash,(char *)mimetype);
		   if (!p) printf("trap: this does not happen\n");
	           value = xmlGetProp(subnode, (const xmlChar *)"priority");
		   if (value){
			p->magic_priority=g_strdup(value);
			g_hash_table_replace(types_hash,g_strdup(mimetype),(gpointer)p);
			   //printf("magic priority=%s ",value);
	   		   g_free(value);
		   }
	           for(ssnode = subnode->children; ssnode; ssnode = ssnode->next){
		      if(xmlStrEqual(ssnode->name, (const xmlChar *)"match")){
			// types: byte, string, big32, host32, little32, big16, host16, little16
		        magic_t *m;
		        int i,j;
		        magic_t **tmp=p->magic;
		        
			type = xmlGetProp(ssnode, (const xmlChar *)"type");
	                value = xmlGetProp(ssnode, (const xmlChar *)"value");//regexp
	                offset = xmlGetProp(ssnode, (const xmlChar *)"offset");
	                mask = xmlGetProp(ssnode, (const xmlChar *)"mask");
		        
			for (i=0;p->magic[i];i++);
		        p->magic = (magic_t **)malloc((i+2)*sizeof(magic_t *));
		        for (j=0;j<i;j++){
			     p->magic[j] = tmp[j];
		        }
		        m = (magic_t *)malloc(sizeof(magic_t));
			if (type) m->type=g_strdup(type); else m->type=NULL;
			if (value) m->value=g_strdup(value);else m->value=NULL;
			if (offset) m->offset=g_strdup(offset);else m->offset=NULL;
			if (mask) m->mask=g_strdup(mask);else m->mask=NULL;
			
		        *(p->magic+i)  = m;
		        *(p->magic+i+1) = NULL;
			g_hash_table_replace(types_hash,g_strdup(mimetype),(gpointer)p);
			
		        g_free(tmp);

			
			//printf("{type=(%s) value=(%s) offset=(%s)}",type,value,offset);
		    	xmlFree(type);
		    	xmlFree(value);
		    	xmlFree(offset);
		    	xmlFree(mask);
		      }
			   
		   }
		   
		}
		if(xmlStrEqual(subnode->name, (const xmlChar *)"comment")){
	   	       			
		    lang2 = xmlGetNsProp(subnode, "lang", XML_NS);
		    if (lang2) {
			//printf("."); fflush(NULL);
		    	xmlFree(lang2);
		    }
		    else {
	               char *e=xmlNodeListGetString(doc, subnode->xmlChildrenNode, 1);
	    	  	//printf("%s",e);
		        xmlFree(e);
		    }
		}
		    
	    }
	    
	    //printf("\n");
	    
	}
	if (!glob_present) g_hash_table_insert(translation_hash, 
			(gpointer) g_strdup(mimetype),
			(gpointer) g_strdup(mimetype));
	g_free(mimetype);
    }
    xmlFreeDoc(doc); 
    g_free(mimefile);
 }

 
    mimefile=g_strconcat("applications.xml-1",NULL);
    if (access(mimefile,R_OK)!=0){
            printf("DBG2: access(%s,R_OK)!=0\n",mimefile);
	g_free(mimefile); mimefile=NULL;

	exit(1);
    }
    xmlKeepBlanksDefault(0);

    if((doc = xmlParseFile(mimefile)) == NULL)
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
	exit(1);
    }
	
    node = xmlDocGetRootElement(doc);
    if(!xmlStrEqual(node->name, (const xmlChar *)"mime_applications"))
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
        xmlFreeDoc(doc);
	exit(1);
    }
   /* Now parse the xml tree */
    for(node = node->children; node; node = node->next)
    {
	xmlChar *id,*application,*arguments;
	if(xmlStrEqual(node->name, (const xmlChar *)"extension"))
	{
	     char *p,*q,*a;
	     id= xmlGetProp(node, (const xmlChar *)"id");
	     application= xmlGetProp(node, (const xmlChar *)"application");
	     arguments= xmlGetProp(node, (const xmlChar *)"arguments");

	     if (!id) g_assert_not_reached();
	     if (!application) g_assert_not_reached();
	     if (*id == '.') p = g_strconcat("*",id,NULL);
	     else p=g_strdup(id);
	     /* '\b', '\f', '\n', '\r', '\t', '\' and '"'*/
	     q=g_strescape ((const gchar *)application,"\f\n\r");
	     if (arguments && strlen(arguments)){
		     char *pp;
		     pp=g_strescape ((const gchar *)arguments,"\f\n\r");
		     a=g_strconcat (q," ",pp,NULL);
		     g_free(pp);
	     } else {
		     a=g_strdup(q);
	     }
	     //printf("app=%s -> %s-> %s\n",application,q,a);
	     g_free(q);
	     mt=(mime_t *)g_hash_table_lookup(glob_hash,p);
	     if (!mt){
		     printf("%s not found in hashtable-->%s!\n",p,a);
	     } else {
		     int i,j;
		     char **tmp=mt->apps;
		     for (i=0;mt->apps[i];i++);
		     mt->apps = (char **)malloc((i+2)*sizeof(char *));
		     for (j=0;j<i;j++){
			     mt->apps[j] = tmp[j];
		     }
		     *(mt->apps+i)  = a;
		     *(mt->apps+i+1) = NULL;
		     g_free(tmp);
		     //printf ("apps is ");
		     //for (i=0;mt->apps[i];i++)printf("[%s] ",mt->apps[i]);
		     
	     }
	     {
		 mime_t *mt2;
		 mt2=g_hash_table_lookup(types_hash,mt->mimetype);
		 if (!mt2) {
			 //printf("%s not found in types_hash!\n",mt->mimetype);
		 } 
		 else {
			 gboolean found=FALSE;
			 char **tmp=mt2->apps;
			 int i,j;
			 for (i=0;mt2->apps[i];i++);
			 
			 for (j=0; j<i ;j++){
				 if (strcmp(mt2->apps[j],a)==0) {found=TRUE; break;}
			 }
			 if (!found) {
			     mt2->apps = (char **)malloc((i+2)*sizeof(char *));
			     for (j=0;j<i;j++) mt2->apps[j] = tmp[j];
			     *(mt2->apps+i)  = g_strdup(a);
			     *(mt2->apps+i+1) = NULL;
			     g_free(tmp);
			 }
		 }
	     }
		     
	     xmlFree(id);
	     xmlFree(application);
	     xmlFree(arguments);

	     
	}
	//printf ("\n");
    }
    
    mimefile=g_strconcat("types.xml-1",NULL);
    if (access(mimefile,R_OK)!=0){
            printf("DBG3: access(%s,R_OK)!=0\n",mimefile);
	g_free(mimefile); mimefile=NULL;

	exit(1);
    }
    xmlKeepBlanksDefault(0);

    if((doc = xmlParseFile(mimefile)) == NULL)
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
	exit(1);
    }
	
    node = xmlDocGetRootElement(doc);
    if(!xmlStrEqual(node->name, (const xmlChar *)"mime_types"))
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
        xmlFreeDoc(doc);
	exit(1);
    }
   /* Now parse the xml tree */
    for(node = node->children; node; node = node->next)
    {
	xmlChar *id,*application,*arguments;
	if(xmlStrEqual(node->name, (const xmlChar *)"type"))
	{
	     char *id,*ext,*key,*type;
	     id= xmlGetProp(node, (const xmlChar *)"id");
	     ext= xmlGetProp(node, (const xmlChar *)"ext");
	     key=g_strconcat("*.",ext,NULL);

	     if (!id) g_assert_not_reached();
	     if (!ext) g_assert_not_reached();
	     
	     mt=(mime_t  *)g_hash_table_lookup(glob_hash,key);
	     if (!mt) mt=(mime_t *)g_hash_table_lookup(glob_hash,ext);
	     if (!mt){
		  printf("%s not found in hashtable!\n",key);
	     } else {
		  g_hash_table_insert(translation_hash, (gpointer) g_strdup(id),(gpointer) g_strdup(mt->mimetype));
	     }
	     xmlFree(id);
	     xmlFree(ext);

	     
	}
    }
    
    for (jj=0; themes[jj];jj++) {
    xmlDocPtr outdoc;

    printf("now doing %s\n",themes[jj]);
    mimefile=g_strconcat(themes[jj],"/","mime.xml-1",NULL);
    if (access(mimefile,R_OK)!=0){
            printf("DBG4: access(%s,R_OK)!=0\n",mimefile);
	g_free(mimefile); mimefile=NULL;

	exit(1);
    }
    xmlKeepBlanksDefault(0);

    if((doc = xmlParseFile(mimefile)) == NULL)
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
	exit(1);
    }
	
    node = xmlDocGetRootElement(doc);
    if(!xmlStrEqual(node->name, (const xmlChar *)"mime_types"))
    {
      	printf("xffm: invalid xml file %s.\n",mimefile);
	g_free(mimefile); mimefile=NULL;
        xmlFreeDoc(doc);
	exit(1);
    }
    g_free(mimefile);
    
   /* Now parse the xml tree */
    for(node = node->children; node; node = node->next)
    {
	xmlChar *id,*application,*arguments;
	if(xmlStrEqual(node->name, (const xmlChar *)"mime"))
	{
	     char *id,*icon,*key,*type;
	     id= xmlGetProp(node, (const xmlChar *)"id");
	     icon= xmlGetProp(node, (const xmlChar *)"icon");
     	     if (!id) g_assert_not_reached();
	     if (!icon) {
		     g_assert_not_reached();
	     }
	     
	     key=(char *)g_hash_table_lookup(translation_hash,id);
	     if (key) mt = (mime_t *)g_hash_table_lookup(types_hash,key);
	     
	     if (!key){
		  printf("%s: %s not found in translation hashtable!\n",themes[jj],id);
	     } 
	     else if (!mt){
		  printf("%s not found in types hashtable!\n",key);
	     } else if (mt) {
		     //printf("updating %s with %s\n",key,icon);
		     mt->icon=g_strdup(icon);
		     g_hash_table_replace (types_hash,g_strdup(key),(gpointer)mt);
       	     }

	     xmlFree(id);
	     xmlFree(icon);

	     
	}
    }
    
    mimefile=g_strconcat(themes[jj],"/","mime.xml",NULL);
    {
       outdoc = xmlNewDoc("1.0");
       outdoc->children = xmlNewDocRawNode(outdoc, NULL, "mime-info", NULL);

       root = (xmlNodePtr) outdoc->children;
       //xmlSetProp(root,"xmlns", "http://www.xfce.org/");
       xmlDocSetRootElement(outdoc, root);
    }   
    g_hash_table_foreach (types_hash, output,NULL);
    
    xfceNS = xmlNewNs (root,"http://www.xfce.org/","xfce");
    if (!xfceNS) g_assert_not_reached();
    xmlSaveFormatFile(mimefile, outdoc, 1);
    g_free(mimefile);

    xmlFreeDoc(outdoc);
    }

    
}



void insert_func2(gpointer key, gpointer value, gpointer user_data){
    int i,j;
    char *k;
    char *kk=(char *)key;
    xmlNodePtr node,subnode;
    mime_t *mt=(mime_t *)value;

    
    if (*kk == '*') kk++;
    if (*kk == '.') kk++;
    k=g_strdup(kk);
    if (strlen(k) && k[strlen(k)-1]=='*') k[strlen(k)-1]=0;
    
    //node = xmlNewTextChild(root, xfceNS, "mime-key", NULL);
    node = xmlNewTextChild(root, NULL, "mime-key", NULL);
    xmlSetProp(node,"key", k);
    
    if (!mt->apps){
	    printf("insert_func2() wrong for %s\n",k);
    } else{
     xmlSetProp(node,"type",mt->mimetype);	    
     for (i=0;mt->apps[i];i++){
      gchar *g=g_strconcat(mt->apps[i]," \%s",NULL);
      subnode = xmlNewTextChild(node, NULL, "application", NULL);
      xmlSetProp(subnode,"command", g);
      g_free(g);
     }
    }
    g_free(k);
    
}
    

void insert_func(gpointer key, gpointer value, gpointer user_data){
    int i;
    char *g=(char *)key;
    xmlNodePtr node,subnode,ssnode;
    mime_t *mt=(mime_t *)value;
    //printf("key=%s\n",g);
    node = xmlNewTextChild(root, NULL, "mime-type", NULL);
    xmlSetProp(node,"type", (char *)key);
    if (!mt->apps){
	    printf("error, !mt->apps with %s at insert_func()\n",(char *)key);
	    return;
    }
    if (mt->magic_priority){
        subnode = xmlNewTextChild(node, NULL, "magic", NULL);
	//printf("priority=%s\n",mt->magic_priority);
        xmlSetProp(subnode,"priority", mt->magic_priority);
	for (i=0;mt->magic[i];i++){
          ssnode = xmlNewTextChild(subnode, NULL, "match", NULL);
          xmlSetProp(ssnode,"type", mt->magic[i]->type);
          xmlSetProp(ssnode,"value", mt->magic[i]->value);
          xmlSetProp(ssnode,"offset", mt->magic[i]->offset);
          if (mt->magic[i]->mask)xmlSetProp(ssnode,"mask", mt->magic[i]->mask);
	}
    }

    
#if 0
    for (i=0;mt->apps[i];i++){
      gchar *g=g_strconcat(mt->apps[i]," \%s",NULL);
      //subnode = xmlNewTextChild(node, xfceNS, "application", NULL);
      subnode = xmlNewTextChild(node, NULL, "application", NULL);
      xmlSetProp(subnode,"command", g);
      g_free(g);
    }
#endif
}


static void reg_write(void){
xmlNodePtr rootM;
    xmlDocPtr doc;
    char *mimefile=NULL;
    
    mimefile=g_strconcat("../applications.xml",NULL);
    
    if (access(mimefile,F_OK)==0 && access(mimefile,W_OK)!=0){
	g_free(mimefile); mimefile=NULL;
	return;
    } 
    
    {
       doc = xmlNewDoc("1.0");
       doc->children = xmlNewDocRawNode(doc, NULL, "mime-info", NULL);

       root = (xmlNodePtr) doc->children;
       //xmlSetProp(root,"xmlns", "http://www.xfce.org/");
       xmlDocSetRootElement(doc, root);
    }   
    xfceNS = xmlNewNs (root,"http://www.xfce.org/","xfce");
    if (!xfceNS) g_assert_not_reached();

    
    /* add additional registers */
    g_hash_table_foreach (glob_hash, insert_func2,NULL);
    //g_hash_table_foreach (types_hash, insert_func,NULL);
    
    /* write xml out */
    
    xmlSaveFormatFile(mimefile, doc, 1);

    xmlFreeDoc(doc);
    g_free(mimefile);
    mimefile=g_strconcat("../types.xml",NULL);
    
    if (access(mimefile,F_OK)==0 && access(mimefile,W_OK)!=0){
	g_free(mimefile); mimefile=NULL;
	return;
    } 
    
    {
       doc = xmlNewDoc("1.0");
       doc->children = xmlNewDocRawNode(doc, NULL, "mime-info", NULL);

       root = (xmlNodePtr) doc->children;
       //xmlSetProp(root,"xmlns", "http://www.xfce.org/");
       xmlDocSetRootElement(doc, root);
    }   
    xfceNS = xmlNewNs (root,"http://www.xfce.org/","xfce");
    if (!xfceNS) g_assert_not_reached();

    
    /* add additional registers */
    //g_hash_table_foreach (glob_hash, insert_func2,NULL);
    g_hash_table_foreach (types_hash, insert_func,NULL);
    
    /* write xml out */
    
    xmlSaveFormatFile(mimefile, doc, 1);

    xmlFreeDoc(doc);
    g_free(mimefile);

    
}

    /*********************/
#if 0    
    mimefile=g_strconcat("mime-types.xml",NULL);
    if (access(mimefile,F_OK)==0 && access(mimefile,W_OK)!=0){
	g_free(mimefile); mimefile=NULL;
	return;
    } 
    
    {
       doc = xmlNewDoc("1.0");
       doc->children = xmlNewDocRawNode(doc, NULL, "mime-info", NULL);

       root = (xmlNodePtr) doc->children;
       xmlSetProp(root,"xmlns", "http://www.xfce.org/");
       xmlDocSetRootElement(doc, root);
    }   
    
    /* add additional registers */

    g_hash_table_foreach (glob_hash, insert_func,NULL);
    /* write xml out */
    
    xmlSaveFormatFile(mimefile, doc, 1);

    xmlFreeDoc(doc);
    g_free(mimefile);
    return;
}
#endif
#if 0
    node = xmlNewTextChild(root, NULL, "mime-type", NULL);

    xmlSetProp(node,"type", mt->mimetype);

    xmlSetProp(node,"id", sfx);
    xmlSetProp(node,"application", reg->app);

    
    if (reg->arg && strlen(reg->arg)){
        xmlSetProp(node, "arguments", reg->arg);
    }

    /* write xml out */
    
    xmlSaveFormatFile(mimefile, doc, 1);

    xmlFreeDoc(doc);
    g_free(mimefile);
    return;
}

#endif

int main(void){
	
	// for namespacing.. xmlNsPtr xfceNs = xmlNewNs (xmlNodePtr node,XFCE_NS, XFCE_NS_PREFIX);



	reg_build_list();
	reg_write();
	return 1;
}


